# Time
import time

# Events
from events import Event

# Players
from players.entity import PlayerEntity
from players.helpers import index_from_userid
from players.helpers import index_from_playerinfo
from players.helpers import edict_from_index

# Filters
from filters.players import PlayerIter

# Engines
from engines.server import engine_server

# Listeners
from listeners.tick import tick_delays

# Stringtables
from stringtables.downloads import Downloadables

# Menus
from menus import SimpleMenu
from menus import Option

# Messages
from messages import SayText2

# Commands
from commands.say import say_command_manager


settings = {}
damage = {}
nospam = {}
flashed = set()


def make_download():
    Downloadables().add("materials/overlays/hitmarker/hitmarkerv2.vmt")
    Downloadables().add("materials/overlays/hitmarker/hitmarkerv2.vtf")

@Event
def game_start(game_event):
    make_download()

@Event
def player_activate(game_event):
    index = index_from_userid(game_event.get_int('userid'))

    if PlayerEntity(index).steamid == 'BOT':
        return

    if index not in settings:
        settings[index] = {}
        settings[index]['hurt'] = 1
        settings[index]['blind'] = 1
        settings[index]['stats'] = 1

@Event
def player_disconnect(game_event):
    index = index_from_userid(game_event.get_int('userid'))
    if index in settings:
        del settings[index]
    if index in damage:
        del damage[index]

@Event
def player_hurt(game_event):
    attacker = game_event.get_int('attacker')
    if not attacker:
        return
    userid   = game_event.get_int('userid')
    dmg      = game_event.get_int('dmg_health')
    index    = index_from_userid(attacker)
    v_index  = index_from_userid(userid)
    player   = PlayerEntity(index)
    v_player = PlayerEntity(v_index)

    if attacker == userid:
        return

    if index in settings:
        if settings[index]['stats'] == 1:
            if index not in damage:
                damage[index] = {}
                damage[index]['attackers'] = {}
                damage[index]['victims'] = {}

            if v_index not in damage[index]['victims']:
                damage[index]['victims'][v_index] = {}
                damage[index]['victims'][v_index]['name'] = v_player.name
                damage[index]['victims'][v_index]['hits'] = 0
                damage[index]['victims'][v_index]['damage'] = 0

            damage[index]['victims'][v_index]['hits'] += 1
            damage[index]['victims'][v_index]['damage'] += dmg

        if settings[index]['hurt'] == 1:
            overlay_on(edict_from_index(index))

    if v_index in settings:
        if settings[v_index]['stats'] == 1:
            if v_index not in damage:
                damage[v_index] = {}
                damage[v_index]['attackers'] = {}
                damage[v_index]['victims'] = {}

            if index not in damage[v_index]['attackers']:
                damage[v_index]['attackers'][index] = {}
                damage[v_index]['attackers'][index]['name'] = player.name
                damage[v_index]['attackers'][index]['hits'] = 0
                damage[v_index]['attackers'][index]['damage'] = 0

            damage[v_index]['attackers'][index]['hits'] += 1
            damage[v_index]['attackers'][index]['damage'] += dmg

@Event
def round_end(game_event):
    for index in PlayerIter('human'):
        if index not in settings:
            continue
        if settings[index]['stats'] == 0:
            continue
        if index not in damage:
            continue

        damage_menu = SimpleMenu()
        damage_menu.append('Damage Stats:')
        damage_menu.append(' ')

        damage_dealt = 0
        hits_dealt = 0
        damage_taken = 0
        hits_taken = 0

        damage_menu.append(Option('Victims:', 1))
        if 'victims' in damage[index] and len(damage[index]['victims']):
            for victim in damage[index]['victims']:
                name = damage[index]['victims'][victim]['name']
                dmg = damage[index]['victims'][victim]['damage']
                hits = damage[index]['victims'][victim]['hits']
                damage_dealt += dmg
                hits_dealt += hits
                damage_menu.append('{0} > Damage: {1} Hits: {2}'.format(name, dmg, hits))
        else:
            damage_menu.append('None')

        damage_menu.append(' ')
        damage_menu.append(Option('Attackers:', 2))

        if 'attackers' in damage[index] and len(damage[index]['attackers']):
            for attacker in damage[index]['attackers']:
                name = damage[index]['attackers'][attacker]['name']
                dmg = damage[index]['attackers'][attacker]['damage']
                hits = damage[index]['attackers'][attacker]['hits']
                damage_taken += dmg
                hits_taken += hits
                damage_menu.append('{0} > Damage: {1} Hits: {2}'.format(name, dmg, hits))
        else:
            damage_menu.append('None')

        damage_menu.append(' ')

        if damage_dealt:
            damage_menu.append('Total Damage Dealt: {0}'.format(damage_dealt))
            damage_menu.append('Total Hits Dealt: {0}'.format(hits_dealt))
            damage_menu.append(' ')
        if damage_taken:
            damage_menu.append('Total Damage Taken: {0}'.format(damage_taken))
            damage_menu.append('Total Hits Taken: {0}'.format(hits_taken))
        
        damage_menu.select_callback = damage_callback
        damage_menu.send(index)

        damage.clear()

        tick_delays.delay(20, damage_menu.close, (index))


@Event
def player_blind(game_event):
    flashed.add(index_from_userid(game_event.get_int('userid')))

@Event
def flashbang_detonate(game_event):
    index = index_from_userid(game_event.get_int('userid'))
    if flashed:
        if len(flashed) == 1 and index in flashed:
            return
        if index in settings and settings[index]['blind'] == 1:
            overlay_on(edict_from_index(index))

        flashed.clear()

def settingsCommand(playerinfo, teamonly, command):
    now = time.time()
    index = index_from_playerinfo(playerinfo)
    if now - nospam.get(index, 0) >= 0.2:
        nospam[index] = now
        if index not in settings:
            settings[index] = {}
            settings[index]['hurt'] = 1
            settings[index]['blind'] = 1
            settings[index]['stats'] = 1

        hurt_text = "[ON]"
        if settings[index]['hurt'] == 0:
            hurt_text = "[OFF]"
        blind_text = "[ON]"
        if settings[index]['blind'] == 0:
            blind_text = "[OFF]"
        stats_text = "[ON]"
        if settings[index]['stats'] == 0:
            stats_text = "[OFF]"

        settings_menu = SimpleMenu()
        settings_menu.append('Hitmarker Settings')
        settings_menu.append(' ')
        settings_menu.append('Display Hitmarker on:')
        settings_menu.append(Option('Player Hurt {0}'.format(hurt_text), 1))
        settings_menu.append(Option('Player Blind {0}'.format(blind_text), 2))
        settings_menu.append(' ')
        settings_menu.append(Option('Show Damage Stats {0}'.format(stats_text), 3))
        settings_menu.append(' ')
        settings_menu.append(Option('Close', 4))
        settings_menu.select_callback = settings_callback
        settings_menu.send(index)
    else:
        SayText2(message="Please do not spam").send(index)

def settings_callback(menu, index, option):
    if option.value == 1:
        if settings[index]['hurt'] == 1:
            settings[index]['hurt'] = 0
        else:
            settings[index]['hurt'] = 1

    elif option.value == 2:
        if settings[index]['blind'] == 1:
            settings[index]['blind'] = 0
        else:
            settings[index]['blind'] = 1

    elif option.value == 3:
        if settings[index]['stats'] == 1:
            settings[index]['stats'] = 0
        else:
            settings[index]['stats'] = 1

    elif option.value == 4:
        menu.close(index)

    if option.value in (1,2,3):
        settingsCommand(PlayerEntity(index).playerinfo, None, None)

def damage_callback(menu, index, option):
    menu.close(index)

def overlay_on(edict):
    engine_server.client_command(edict, "r_screenoverlay overlays/hitmarker/hitmarkerv2")
    tick_delays.delay(1, overlay_off, (edict))

def overlay_off(edict):
    engine_server.client_command(edict, "r_screenoverlay 0")

def load():
    make_download()
    say_command_manager.register_commands('!settings', settingsCommand)

def unload():
    say_command_manager.unregister_commands('!settings', settingsCommand)
